require "world"
require "player"
require "levels"
require "menu"


DEBUG = false
state = ""
swapTimer = 0
TIMEMOD = 1

pixelTextS = love.graphics.newFont("font/roboto.ttf", 18)
pixelTextL = love.graphics.newFont("font/roboto.ttf", 72)
levelText = love.graphics.newFont("font/roboto.ttf", 36)

screen = {}
screen.w = love.window.getWidth()
screen.h = love.window.getHeight()

playButton = {x = (screen.w/2) - 100, y = (screen.h/2) - 50, w = 200, h = 100}
pausedBox = {x = (screen.w/2) - 100, y = (screen.h/2) - 50, w = 285, h = 100}

function love.load()
	setupShaders()
	SetUpCamera()
	love.graphics.setFont(pixelTextS)
	love.graphics.setBackgroundColor(54,69,79)
	state = "menu"
	restart()
end

function love.update(dt)
	if state == "playing" then
		player_update(dt * TIMEMOD)
		world_update(dt * TIMEMOD)

		cam.tx = -(player.x + player.xvel/2) + screen.w/2
		cam.ty = -player.y + -player.yvel/2 + screen.h/2

		cam.x = cam.x + ((cam.tx - cam.x) * cam.speed * dt)
		cam.y = cam.y + ((cam.ty - cam.y) * cam.speed * dt)

		if swapTimer > 0 then
			TIMEMOD = 0
		else
			TIMEMOD = 1
		end

		pauseBarH = (swapTimer / 90) * 40

		if swapTimer > 0 then
			swapTimer = swapTimer - 1
		end
	end
end

function love.draw()
	if state == "playing" then
		love.graphics.setShader(BnW)
		love.graphics.push()
		love.graphics.translate(cam.x, cam.y)
		world_draw()
		player_draw()

		love.graphics.setColor(207,207,196)
		love.graphics.rectangle("fill", players[pSel].x - players[pSel].w/2 - 6, players[pSel].y + players[pSel].h/2, 6, -pauseBarH)
		love.graphics.setColor(255,255,255)

		if DEBUG == true then
			for i,v in ipairs(zones) do
				for k,b in ipairs(players) do
					if b.id == v.id then
						love.graphics.setColor(255,255,255)
						love.graphics.line(b.x, b.y, v.x + v.w/2, v.y + v.h/2)
					end

					love.graphics.setColor(0,0,0)
					love.graphics.rectangle("line",b.x - b.w/2, b.y - b.h/2, b.w, b.h)
					love.graphics.rectangle("line",v.x, v.y, v.w, v.h)
				end

				if v.playerIn == true then
					love.graphics.setColor(0,0,0)
					love.graphics.circle("fill",v.x,v.y,4)
				else
					love.graphics.setColor(0,0,0)
					love.graphics.circle("line",v.x,v.y,4)
				end
			end
		end
		love.graphics.pop()

		if DEBUG == true then
			love.graphics.setColor(255,255,255)
			love.graphics.print("xPos: "..round(player.x,1), 10,10)
			love.graphics.print("yPos: "..round(player.y,1), 10,30)
			love.graphics.print("xVel: "..round(player.xvel,1), 10,50)
			love.graphics.print("yVel: "..round(player.yvel,1), 10,70)
			love.graphics.print("TIMEMOD: "..TIMEMOD, 10,90)
			love.graphics.print("winTimer: "..winTimer, 10, 110)
		end
	elseif state == "menu"
		then
		menuDraw()
	elseif state == "paused"
		then
		love.graphics.push()
		love.graphics.translate(cam.x,cam.y)
		world_draw()
		player_draw()
		love.graphics.pop()
		pausedDraw()
	elseif state == "end" then
		world_draw()
	end
end

function love.keypressed(key)
	player_keypress(key)

	if key == "escape" and state == "playing"
		then
		state = "paused"
	elseif state == "paused" and key == "escape"
		then
		state = "playing"
	end

	if key == "f1"
		then
		love.event.quit()
	end

	if key == "`" then
		DEBUG = not DEBUG
	end
end

function love.mousepressed( x, y, button )
	menuUpdate(x,y,button)
end

function SetUpCamera()
	screen = {}
	screen.w = love.window.getWidth()
	screen.h = love.window.getHeight()

	cam = {}
	cam.x = 0
	cam.y = 0
	cam.tx = 0
	cam.ty = 0
	cam.pauseScale = 0.5

	cam.speed = 3
end

function setupShaders()
	bloom2 = love.graphics.newShader[[
	   //BlackBulletIV
	   extern vec2 size = vec2(20,20);
	   extern int samples = 20; // pixels per axis; higher = bigger glow, worse performance
	   extern float quality = .15; // lower = smaller glow, better quality

	   vec4 effect(vec4 colour, Image tex, vec2 tc, vec2 sc)
	   {
	      vec4 source = Texel(tex, tc);
	      vec4 sum = vec4(0);
	      int diff = (samples - 1) / 2;
	      vec2 sizeFactor = vec2(1) / size * quality;

	      for (int x = -diff; x <= diff; x++)
	      {
	         for (int y = -diff; y <= diff; y++)
	         {
	            vec2 offset = vec2(x, y) * sizeFactor;
	            sum += Texel(tex, tc + offset);
	         }
	      }

	   return ((sum / (samples * samples)) + source) * colour;
	   }
	]]
end

function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end